(*****************************************************************************)
(*  robot_display.ml :                                                       *)
(*                                                                           *)
(* POD : E.Chailloux  P.Manoury  B.Pagano                                    *)
(*****************************************************************************)

(*************************************************************************)
(*                       ROBOT ET MONDE AFFICHABLES                      *)
(*************************************************************************)

open Robot_def

class virtual ['a] mondeV_displayV =
  object 
    method virtual get_dim : int * int
    method virtual get_roblist : 'a  list
    method virtual display : unit
  end

class virtual ['a] robotV_displayV =
  object
    method virtual get_pos : int * int 
    method virtual get_speed : int * int 
    method virtual display : unit
  end 





class virtual ['a] mondeV_txt () =
  object (s)
    inherit ['a] mondeV_displayV  
    method display = 
      (let (h,l) = s#get_dim in Printf.printf "\nLE MONDE :  %d x %d\n" h l) ;
      List.iter (fun r -> r#display) s#get_roblist ;
      flush stdout 
  end


class virtual ['a] robotV_txt ident = 
  object (s)
    inherit ['a] robotV_displayV 
    val id = ident 
    method display = 
      let (x,y) = s#get_pos 
      in Printf.printf "  Robot (%s) : %d - %d\n" id x y 
  end







open Graphics
open Graphic_server 

class virtual ['a] mondeV_graph (gc:enhanced_graph_client) = 
    object (s)
    inherit ['a] mondeV_displayV

    initializer
      gc#send clear ; 
      let (h,l) = s#get_dim in 
      for i=0 to h do gc#send (line black (0,i*25) (l*25,i*25)) done;
      for i=0 to l do gc#send (line black (i*25,0) (i*25,h*25)) done;

    method display = List.iter (fun r -> r#display) s#get_roblist ;
end


class virtual ['a] robotV_graph ident c (gc:enhanced_graph_client) = 
  object (s)
    inherit ['a] robotV_displayV
    val mutable lx = -1
    val mutable ly = -1
    val mutable lvx = 0
    val mutable lvy = 0
    val mutable col = c
    method get_col = col 
    method set_col c = col <- c
    method display = 
      let (x,y) = s#get_pos and (vx,vy) = s#get_speed 
      in  
	if (x=lx && y=ly && vx=lvx && vy=lvy) then () else 
	( if lx <> -1 then gc#send (disk background (lx*25+12,ly*25+12) 10) );
	gc#send (disk col (x*25+12,y*25+12) 10) ;
	gc#send (line black (x*25+12,y*25+12) (x*25+12+vx*6,y*25+12+vy*6)) ;
	lx <- x ; ly <- y ; lvx <- lvy  	 
  end



(* les robots qui s'affichent en texte *)


class ['a] ferme_txt hi li = 
  object 
  inherit ['a] monde_ferme hi li 
  inherit ['a] mondeV_txt ()
end

class ['a] poli_txt id p v = 
  object
    inherit ['a] robot_poli p v 
    inherit ['a] robotV_txt id 
end

class ['a] fixe_txt id p = 
  object
    inherit ['a] robot_fixe p 
    inherit ['a] robotV_txt id 
end

class ['a] fou_txt id p = 
  object
    inherit ['a] robot_fou p 
    inherit ['a] robotV_txt id 
end

class ['a] amical_txt id p v a= 
  object
    inherit ['a] robot_amical p v a
    inherit ['a] robotV_txt id 
end

class ['a] fou2_txt id p v = 
  object
    inherit ['a] robot_fou_inertie p v 
    inherit ['a] robotV_txt id 
end





(* les robots qui s'affichent avec un enhanced_graph_client *)


class ['a] poli_graph id c p v gc = 
  object
    inherit ['a] robot_poli p v 
    inherit ['a] robotV_graph id c gc  
end

class ['a] fixe_graph id c p gc = 
  object
    inherit ['a] robot_fixe p 
    inherit ['a] robotV_graph id c gc  
end

class ['a] fou_graph id c p gc = 
  object
    inherit ['a] robot_fou p 
    inherit ['a] robotV_graph id c gc   
end

class ['a] amical_graph id c p v a gc = 
  object (s)
    inherit ['a] robot_amical p v a
    inherit ['a] robotV_graph id c gc as super
    method display = 
      let (x,y)=s#get_pos
      in super#display ; gc#send (disk ami#get_col (x*25+12,y*25+12) 4) 
end

class ['a] fou2_graph id c p v gc = 
  object
    inherit ['a] robot_fou_inertie p v 
    inherit ['a] robotV_graph id c gc
end

class ['a] presse_graph id c p v gc = 
  object
    inherit ['a] robot_presse p v 
    inherit ['a] robotV_graph id c gc
end
